Option Explicit On 
Option Strict On
Option Compare Binary

Imports System
Imports System.Windows.Forms

' <remarks>
'   Hauptformular der Anwendung.
' </remarks>
Public Class MainForm
    Inherits System.Windows.Forms.Form

#Region " Delegat-Deklarationen "
    Private Delegate Function SubWndProcDelegate( _
        ByVal hWnd As IntPtr, _
        ByVal msg As Int32, _
        ByVal wParam As Int32, _
        ByVal lParam As Int32) As Int32
#End Region

#Region " API-Deklarationen "

    ' Um die Mglichkeit offen zu lassen, auch einen "echten" Funktionszeiger 
    ' in Form einer Variablen vom Typ IntPtr an das API zu bergeben, wird der 
    ' Name SetWindowLong berladen.
    Private Declare Function SetWindowLong _
        Lib "user32.dll" Alias "SetWindowLongA" ( _
            ByVal hWnd As IntPtr, _
            ByVal nIndex As Int32, _
            ByVal dwNewLong As IntPtr) As Int32
    Private Declare Function SetWindowLong _
        Lib "user32.dll" Alias "SetWindowLongA" ( _
            ByVal hWnd As IntPtr, _
            ByVal nIndex As Int32, _
            ByVal dwNewLong As SubWndProcDelegate) As IntPtr

    Private Declare Function CallWindowProc _
        Lib "user32.dll" Alias "CallWindowProcA" ( _
            ByVal lpPrevWndFunc As IntPtr, _
            ByVal hWnd As IntPtr, _
            ByVal msg As Int32, _
            ByVal wParam As Int32, _
            ByVal lParam As Int32) As Int32

    Private Const GWL_WNDPROC As Int32 = -&H4

    Private Const WM_NCLBUTTONDBLCLK As Int32 = &HA3
    Private Const WM_NCLBUTTONDOWN As Int32 = &HA1
    Private Const WM_NCLBUTTONUP As Int32 = &HA2
#End Region

    Private m_intptrPrevWndProc As IntPtr

#Region " Windows Form Designer generated code "
    Public Sub New()
        MyBase.New()
        InitializeComponent()
    End Sub

    Protected Overloads Overrides Sub Dispose(ByVal disposing As Boolean)
        If disposing Then
            If Not components Is Nothing Then
                components.Dispose()
            End If
        End If

        ' Subclassing beenden.
        TerminateSubclassing(Me.Handle)

        MyBase.Dispose(disposing)
    End Sub

    Private components As System.ComponentModel.IContainer

    Friend WithEvents Label1 As System.Windows.Forms.Label
    Friend WithEvents chkNoWindowStateChange As System.Windows.Forms.CheckBox

    <System.Diagnostics.DebuggerStepThrough()> _
    Private Sub InitializeComponent()
        Me.Label1 = New System.Windows.Forms.Label()
        Me.chkNoWindowStateChange = New System.Windows.Forms.CheckBox()
        Me.SuspendLayout()
        '
        'Label1
        '
        Me.Label1.Location = New System.Drawing.Point(16, 16)
        Me.Label1.Name = "Label1"
        Me.Label1.Size = New System.Drawing.Size(288, 32)
        Me.Label1.TabIndex = 0
        Me.Label1.Text = "Doppelklicken Sie den Non-Client-Bereich des Formulars (z.B. die Titelleiste)."
        '
        'chkNoWindowStateChange
        '
        Me.chkNoWindowStateChange.Location = New System.Drawing.Point(16, 56)
        Me.chkNoWindowStateChange.Name = "chkNoWindowStateChange"
        Me.chkNoWindowStateChange.Size = New System.Drawing.Size(288, 32)
        Me.chkNoWindowStateChange.TabIndex = 0
        Me.chkNoWindowStateChange.Text = "&nderung des Fensterstatus bei Doppelklick auf den Non-Client-Bereich des Formul" & _
        "ars unterbinden"
        '
        'MainForm
        '
        Me.AutoScaleBaseSize = New System.Drawing.Size(5, 13)
        Me.ClientSize = New System.Drawing.Size(320, 246)
        Me.Controls.AddRange(New System.Windows.Forms.Control() {Me.chkNoWindowStateChange, Me.Label1})
        Me.Name = "MainForm"
        Me.StartPosition = System.Windows.Forms.FormStartPosition.CenterScreen
        Me.Text = "SubclassingOldWay"
        Me.ResumeLayout(False)

    End Sub
#End Region

    Private Sub MainForm_Load( _
      ByVal sender As System.Object, _
      ByVal e As System.EventArgs) _
      Handles MyBase.Load

        ' Subclassing starten.
        InitializeSubclassing(Me.Handle)
    End Sub

    ' <summary>
    '   Initialisieren des Subclassing.
    ' </summary>
    ' <param name="hWnd">Handle des zu 
    '   subclassenden Fensters.</param>
    Public Sub InitializeSubclassing( _
      ByVal hWnd As IntPtr)
        m_intptrPrevWndProc = SetWindowLong( _
          hWnd, _
          GWL_WNDPROC, _
          AddressOf SubWndProc)
    End Sub

    ' <summary>
    '   Beenden des Subclassing durch 
    '   Zurcksetzen des Funktionszeigers 
    '   auf die echte Fensterprozedur.
    ' </summary>
    ' <param name="hWnd">Handle des Fensters, 
    '   dessen Subclassing entfernt werden soll.</param>
    Public Sub TerminateSubclassing( _
      ByVal hWnd As IntPtr)
        SetWindowLong(hWnd, _
          GWL_WNDPROC, _
          m_intptrPrevWndProc)
    End Sub

    ' <summary>
    '   Unsere Fensterprozedur, hier wird die 
    '   Doppelklick-Meldung auf dem Non-Client-Bereich
    '   des Formulars angefangen.
    ' </summary>
    ' <param name="hWnd">Siehe Dokumentation zum Platform SDK.</param>
    ' <param name="msg">Siehe Dokumentation zum Platform SDK.</param>
    ' <param name="wParam">Siehe Dokumentation zum Platform SDK.</param>
    ' <param name="lParam">Siehe Dokumentation zum Platform SDK.</param>
    ' <returns>Siehe Dokumentation zum Platform SDK.</returns>
    Private Function SubWndProc( _
      ByVal hWnd As IntPtr, _
      ByVal msg As Int32, _
      ByVal wParam As Int32, _
      ByVal lParam As Int32) As Int32

        ' Die Meldung besagt, dass der 
        ' Non-Client-Bereich des Formulars doppelt
        ' angeklickt wurde.
        If msg = WM_NCLBUTTONDBLCLK Then
            MessageBox.Show( _
              "Der Non-Client-Bereich wurde doppelt angeklickt.", _
              "SubWndProc")

            ' Wenn der Benutzer will, dass die Meldung 
            ' unterdrckt wird, dann steigen
            ' wir hier aus, ohne sie an Windows weiterzugeben.
            If chkNoWindowStateChange.Checked Then
                Return 0
            End If
        End If

        ' Weiterreichen der Messages an Windows. 
        ' Hier besteht auch die Mglichkeit,
        ' Meldungen zu unterdrcken, indem man 
        ' sie nicht weiterleitet.
        Return CallWindowProc(m_intptrPrevWndProc, hWnd, msg, wParam, lParam)
    End Function
End Class